#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <wiringPi.h>
#include "arduino.h"
#include "circle.h"
#include "io.h"
#include "request.h"
#include "routine.h"
#include "uss.h"
#include "camera.h"
#include "flag.h"
#include "key_watch.h"
#include "io.h"
#include "speak.h"
#include "timer.h"
#include "timer2.h"
#include "light.h"

#define ACCESS_TOKEN "-------------------"
#define MESSAGE_EMERGENCY "非常停止スイッチが押されたよ"
#define MESSAGE_BATTERY_LOW "モーターバッテリーの残量が少ないよ"


int main(){

	enum {SEARCH, TRASH_DETECTION, HOME} mode = SEARCH;

	double volt;
	char line_emergency[200];
	char line_battery_low[200];

	printf("Pilot program start !!!\n");

	clear_state_flag();
	clear_run_flag();
	clear_state();

	if(io_open() < 0){
		printf("failed to open GPIO device\n");
		return -1;
	}
	if(arduino_open() < 0){
		//printf("failed to open Arduino\n");
		return -1;
	}
	if(uss_open() < 0){
		printf("failed to open USS device\n");
	}

	if(camera_set(0) < 0){
		printf("failed to open camera 0 \n");
		return -1;
	}

	request_get_batt(&volt);
	usleep(500*1000);
	request_get_batt(&volt);
	usleep(500*1000);
	request_get_batt(&volt);
	printf("バッテリー残量：%f[V]\n",volt);
	if(volt <= 0){
		printf("駆動用バッテリーが接続されてないよ\n");
		while(volt <= 0){
			request_get_batt(&volt);
			usleep(1000);
		}
	}
	if(volt <= 7.2){
		printf("駆動用バッテリーを交換してね\n");
		goto end;
	}

	sprintf(line_emergency, "curl -X POST -H 'Authorization: Bearer %s ' -F 'message= %s ' https://notify-api.line.me/api/notify",ACCESS_TOKEN, MESSAGE_EMERGENCY);
	sprintf(line_battery_low, "curl -X POST -H 'Authorization: Bearer %s ' -F 'message= %s ' https://notify-api.line.me/api/notify",ACCESS_TOKEN, MESSAGE_BATTERY_LOW);
	system("uvcdynctrl -L mirs.gpfl");

	//キー入力を待つ
	printf("press enter to start\n");
	getchar();

	//初期状態の設定
	mode = SEARCH;

	//赤丸検出の設定
	circle_camera_set();

	//赤丸認識スレッドの生成
	circle_create();

	//超音波スレッド生成
	uss_create();

	//キー監視スレッド生成
	key_watch_create();

	//battery_create();

	//タイマースレッド生成
	//timer_srd_create();


	sleep(1);

	//request_set_runmode(STR, 20, 45);
	//sleep(5);
	//request_set_runmode(ROT, 30, 90);
	//sleep(5);

	while(1){

		if( circle_flag == 0 ) mode = SEARCH;
		if( circle_flag == 1 ) mode = TRASH_DETECTION;
		if( escape_flag == 1 ) mode = HOME;
		
		switch(mode){
			case SEARCH:
				routine_search();
				break;
			case TRASH_DETECTION:
				routine_trash_detection();
				break;
			case HOME:
				printf("\n帰還開始\n");
				request_set_runmode(STP, 0, 0);
				sleep(3);
				routine_home(course_number, dist_last);
				//request_set_runmode(STR, 20, -45);
				//sleep(3);
				printf("帰還完了\n");
				while(state == 0){
					sleep(1);
				}
				printf("休み時間だよ\n");
				request_set_runmode(ROT, 30, 90);
				sleep(4);
				escape_flag = 0;
				break;
				//goto end;
		}
		if( quit_flag == 1) break;
		//if( state == 0) break;
		if( battery_flag == 1) {
			system(line_emergency);
			usleep(1000);
			goto end;
		}
		if( battery_flag == 2) {
			system(line_battery_low);
			usleep(1000);
			goto end;
		}
		usleep(10*1000);

	}
	
	// pilot 終了処理
end:
	printf("\nおわり\n");
	request_set_runmode(STP, 0, 0);	

	//speak_cancel();
	circle_cancel();
	//uss_cancel();
	key_watch_cancel();
	timer_cancel();
	timer2_cancel();

	camera_release();
	arduino_close();
	
	return 0;
}
