#include <stdio.h>
#include <unistd.h>
#include <pthread.h>
#include <wiringPiI2C.h>
#include "uss.h"

// 測定の時間間隔[ms]
static const int t_uss = 50;

// 測定範囲の下限・上限[cm]
static const int uss_min =  16;
static const int uss_max = 600;

// センサからMIRS中心までの距離[cm]
static const int dist_center = 0;

static int fd_f, fd_b, fd_l, fd_r;

static void *thread_code(void*);
static pthread_t task_id;

static long uss_f, uss_b, uss_l;
static long uss_r = 10;

// 障害物のしきい値
static int threshold = 25;

void *thread_code(void *t){
        // 実行周期[ms]
        const int cycle = 20 ; 

        while(1){
                uss_f = uss_get_f();
                usleep(cycle * 1000);
                uss_b = uss_get_b();
                usleep(cycle * 1000);
                uss_l = uss_get_l();
                usleep(cycle * 1000);
                uss_r = uss_get_r();
                usleep(cycle * 1000);
        }

        return 0;
}

int uss_create(){

	int policy;
        struct sched_param param;

        // スレッドの生成
        if(pthread_create(&task_id, NULL, thread_code, NULL) != 0){
                printf("failed to create uss thread\n");
                return -1;
        }

        // スレッドパラメータの確認
        pthread_getschedparam(task_id, &policy, &param);
        printf("uss thread created (plicy = %s, priority = %d)\n",
                ((policy == SCHED_FIFO) ? "SCHED_FIFO" : (policy == SCHED_RR) ? "SCHED_RR" : (policy == SCHED_OTHER) ? "SCHED_OTHER" : "???"),
                param.sched_priority);

        return 0;
}

void uss_get_val(int *usd_f, int *usd_b, int *usd_l, int *usd_r){

	*usd_f = uss_f;
	*usd_b = uss_b;
	*usd_l = uss_l;
	*usd_r = uss_r;

}

int uss_detection(int *usd_f, int *usd_b, int *usd_l, int *usd_r){

	*usd_f = uss_f;
	*usd_b = uss_b;
	*usd_l = uss_l;
	*usd_r = uss_r;
	
	if(uss_f < threshold || uss_l < threshold || uss_r < threshold){
		return 1; //前に障害物がある
	}
	if(uss_b < threshold){
		return 2;
	}

	return 0;
}

int uss_open(){

	if(uss_open_f() != 0){
		printf(" forward uss open faild !!\n");
		return -1;
	}
	if(uss_open_b() != 0){
		printf(" back uss open faild !!\n");
		return -1;
	}
	if(uss_open_l() != 0){
		printf(" left uss open faild !!\n");
		return -1;
	}
	if(uss_open_r() != 0){
		printf(" right uss open faild !!\n");
		return -1;
	}
	return 0;
}

int uss_open_f(){
	fd_f = wiringPiI2CSetup(0x70);
	
	// ソフトウェアリビジョンの確認
	if(wiringPiI2CReadReg8(fd_f, 0x00) == 0x06){
		printf("USS_F opened\n");
		return 0;
	}else{
		printf("failed to open USS_F\n");
		return -1;
	}
}

int uss_open_b(){
	fd_b = wiringPiI2CSetup(0x71);
	
	// ソフトウェアリビジョンの確認
	if(wiringPiI2CReadReg8(fd_b, 0x00) == 0x06){
		printf("USS_B opened\n");
		return 0;
	}else{
		printf("failed to open USS_B\n");
		return -1;
	}
}

int uss_open_l(){
	fd_l = wiringPiI2CSetup(0x72);
	
	// ソフトウェアリビジョンの確認
	if(wiringPiI2CReadReg8(fd_l, 0x00) == 0x06){
		printf("USS_L opened\n");
		return 0;
	}else{
		printf("failed to open USS_L\n");
		return -1;
	}
}

int uss_open_r(){
	fd_r = wiringPiI2CSetup(0x73);
	
	// ソフトウェアリビジョンの確認
	if(wiringPiI2CReadReg8(fd_r, 0x00) == 0x06){
		printf("USS_R opened\n");
		return 0;
	}else{
		printf("failed to open USS_R\n");
		return -1;
	}
}

long uss_get_f(){
	unsigned char lowbyte, highbyte;
	long val;
	
	wiringPiI2CWriteReg8(fd_f, 0x00, 0x51);
	usleep(t_uss * 1000);
	
	lowbyte  = wiringPiI2CReadReg8(fd_f, 0x03);
	highbyte = wiringPiI2CReadReg8(fd_f, 0x02);
	val = lowbyte + highbyte * 256;
	
	if(val < uss_min){
		return -1;
	}else{
		return val + dist_center;
	}
}

long uss_get_b(){
	unsigned char lowbyte, highbyte;
	long val;
	
	wiringPiI2CWriteReg8(fd_b, 0x00, 0x51);
	usleep(t_uss * 1000);
	
	lowbyte  = wiringPiI2CReadReg8(fd_b, 0x03);
	highbyte = wiringPiI2CReadReg8(fd_b, 0x02);
	val = lowbyte + highbyte * 256;
	
	if(val < uss_min){
		return -1;
	}else{
		return val + dist_center;
	}
}

long uss_get_l(){
	unsigned char lowbyte, highbyte;
	long val;
	
	wiringPiI2CWriteReg8(fd_l, 0x00, 0x51);
	usleep(t_uss * 1000);
	
	lowbyte  = wiringPiI2CReadReg8(fd_l, 0x03);
	highbyte = wiringPiI2CReadReg8(fd_l, 0x02);
	val = lowbyte + highbyte * 256;
	
	if(val < uss_min){
		return -1;
	}else{
		return val + dist_center;
	}
}

long uss_get_r(){
	unsigned char lowbyte, highbyte;
	long val;
	
	wiringPiI2CWriteReg8(fd_r, 0x00, 0x51);
	usleep(t_uss * 1000);
	
	lowbyte  = wiringPiI2CReadReg8(fd_r, 0x03);
	highbyte = wiringPiI2CReadReg8(fd_r, 0x02);
	val = lowbyte + highbyte * 256;
	
	if(val < uss_min){
		return -1;
	}else{
		return val +dist_center;
	}

}
